from burlap.locale import LocalesSatchel
from burlap.tests.base import TestCase

class LocaleTests(TestCase):

    def test_get_locale_dict(self):
        s = LocalesSatchel()

        text = '''#  File generated by update-locale
LANG="en_US"
LANGUAGE="en_US:en"
'''

        self.assertEqual(s.get_locale_dict(text), {'LANG': "en_US", 'LANGUAGE': "en_US:en"})
